/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Person.java 5369 2010-02-24 14:58:19Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.mavenplugin.examples.directory;

import java.io.Serializable;
import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;

/**
 * Entity bean for table person.
 * @author Julien Blais
 */
@Entity
@Table(name = "PERSON")
@NamedQueries({
@NamedQuery(name = "Person.findAll", query = "SELECT p FROM Person p"),
@NamedQuery(name = "Person.findByIdpers", query = "SELECT p FROM Person p WHERE p.idpers = :idpers"),
@NamedQuery(name = "Person.findByFirstname", query = "SELECT p FROM Person p WHERE p.firstname = :firstname"),
@NamedQuery(name = "Person.findByLastname", query = "SELECT p FROM Person p WHERE p.lastname = :lastname"),
@NamedQuery(name = "Person.findByAdress", query = "SELECT p FROM Person p WHERE p.adress = :adress"),
@NamedQuery(name = "Person.findByPhone", query = "SELECT p FROM Person p WHERE p.phone = :phone")})
public class Person implements Serializable {
    /**
     * Id for serializable class.
     */
    private static final long serialVersionUID = 124893676373788L;

    /**
     * Auto generated id of this instance.
     */
    @Id
    @Basic(optional = false)
    @Column(name = "IDPERS")
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Integer idpers;

    /**
     * First name of the person.
     */
    @Basic(optional = false)
    @Column(name = "FIRSTNAME")
    private String firstname;

    /**
     * Last name of the person.
     */
    @Column(name = "LASTNAME")
    private String lastname;

    /**
     * Adress of the person.
     */
    @Column(name = "ADRESS")
    private String adress;

    /**
     * Phone number of the person.
     */
    @Basic(optional = false)
    @Column(name = "PHONE")
    private String phone;

    /**
     * Empty constructor.
     */
    public Person() {
    }

    /**
     * Create a new Person.
     * @param firstname the first name of the person
     * @param phone the phone number of the person
     */
    public Person(final String firstname, final String phone) {
        this.firstname = firstname;
        this.phone = phone;
    }

    /**
     * Get the id of this instance.
     * @return the person id
     */
    public Integer getIdpers() {
        return idpers;
    }

    /**
     * Gets the first name of the person.
     * @return the first name
     */
    public String getFirstname() {
        return firstname;
    }

    /**
     * Set the first name of the person.
     * @param firstname the new first name
     */
    public void setFirstname(final String firstname) {
        this.firstname = firstname;
    }

    /**
     * Get the last name of the person.
     * @return the last name
     */
    public String getLastname() {
        return lastname;
    }

    /**
     * Set the last name of the person.
     * @param lastname the new last name
     */
    public void setLastname(final String lastname) {
        this.lastname = lastname;
    }

    /**
     * Gets the adress of the person.
     * @return the adress of the person
     */
    public String getAdress() {
        return adress;
    }

    /**
     * Set the adress of the person.
     * @param adress the new adress of the person
     */
    public void setAdress(final String adress) {
        this.adress = adress;
    }

    /**
     * Get the phone number of the person.
     * @return the phone number
     */
    public String getPhone() {
        return phone;
    }

    /**
     * Set the phone number of the person.
     * @param phone the new phone number
     */
    public void setPhone(final String phone) {
        this.phone = phone;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        if (idpers != null) {
            hash += idpers.hashCode();
        } else {
            hash += 0;
        }
        return hash;
    }

    @Override
    public boolean equals(final Object object) {

        if (!(object instanceof Person)) {
            return false;
        }
        Person other = (Person) object;
        if ((this.idpers == null && other.idpers != null) || (this.idpers != null && !this.idpers.equals(other.idpers))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "Person[idpers=" + idpers + "]";
    }

}
